/*
 * Animator.java
 *
 * Created on September 14, 2005, 10:46 PM
 *
 * CS101 Animation classes
 * (c)2005 Olin College
 */

package cs101.anim;

/**
 * Animates an {@link Animate Animate} class.
 *
 * @author Mike Wessler
 */
public class Animator {
    public static enum StartOption {START_NOW, START_LATER};
    
    /** default minimum delay time for this animator (0.1 sec) */
    public static final float DEFAULT_DURATION = 0.1f;
    
    /** default delay time range for this animator (0 sec) */
    public static final float DEFAULT_RANGE = 0;

    /** the object to animate */
    private Animate anim;
    
    /** the minimum animation pause, in milliseconds */
    private long period;
    
    /** the range of extra random delay to add, in milliseconds */
    private long range;
    
    /** the animation thread */
    private Thread animating;
        
    /**
     * Creates a new instance of Animator.
     * @param anim the Animate object to animate
     */
    public Animator(Animate anim) {
        this(anim, StartOption.START_NOW, DEFAULT_DURATION, DEFAULT_RANGE);
    }
    
    /**
     * Creates a new instance of Animator.
     * @param anim the Animate object to animate
     * @param when a StartOption describing whether to start this thread
     * immediately.  One of StartOption.START_NOW or StartOption.START_LATER.
     */
    public Animator(Animate anim, StartOption when) {
        this(anim, when, DEFAULT_DURATION, DEFAULT_RANGE);
    }
    
    /**
     * Creates a new instance of Animator.
     * @param anim the Animate object to animate
     * @param when a StartOption describing whether to start this thread
     * immediately
     * @param period the minimum time between calls to the Animate's act
     * method
     */
    public Animator(Animate anim, StartOption when, float period) {
        this(anim, when, period, DEFAULT_RANGE);
    }
    
    /**
     * Creates a new instance of Animator.
     * @param anim the Animate object to animate
     * @param when a StartOption describing whether to start this thread
     * immediately
     * @param period the minimum time between calls to the Animate's act
     * method
     * @param variance the range of random time to add between calls to the
     * Animate's act method
     */
    public Animator(Animate anim, StartOption when,
            float period, float variance)
    {
        this.anim = anim;
        this.period = (long)(period*1000);
        this.range = (long)(variance*1000);
        if (when == StartOption.START_NOW) {
            start();
        }
    }
    
    /**
     * Starts (or restarts) this Animator.  If start is called on an
     * already-running Animator, the Animate client's act() method is called
     * immediately, and may run concurrently with the previously invoked act()
     * method.
     */
    public void start() {
        // create a new thread for this run.
        // use an inner class so Animator doesn't have to implement Runnable.
        animating = new Thread(new Runnable() {
            public void run() {
                while (animating == Thread.currentThread()) {
                    anim.act();
                    try {
                        Thread.sleep(period+(int)(Math.random()*range));
                    } catch (InterruptedException ie) {}
                }
            }
        });
        animating.start();
    }
    
    /**
     * Stops the Animator.  If an act() call is underway, the call will finish,
     * and act() will not be called again.  Has no effect if the Animator has
     * already stopped.
     */
    public void stop() {
        Thread hold = animating;
        animating = null;
        if (hold != null) {
            hold.interrupt();
        }
    }
    
    /**
     * Gets whether this Animator is animating or not
     */
    public boolean isAnimating() {
        return animating != null;
    }
}
